#include <extdll.h>			// always
#include <meta_api.h>		// of course
#include <dllapi.h>
#include "sdk_util.h"		// UTIL_LogPrintf, etc
#include <h_export.h>
#include "plugin.h"
#include "cbase.h"
#include <usercmd.h>
#include <pm_defs.h>
#include <enginecallback.h>

#define M_INT(a)		(*(int *)a)
#define M_FLOAT(a)		(*(float *)a)
#define M_STRING(a)		((char *)a)

void (*MsgFunction)(void *) = NULL;

enginefuncs_t g_engfuncs;
globalvars_t  *gpGlobals;
playerdata_t PlayerData[33];

bool g_bRegMsg = false;
float g_flGameTime = 0;

void AdvStatus ( void )
{
	int iPing = 0;
	int iLoss = 0;
		
	UTIL_LogPrintf("STARTUSERDATA\n");
	UTIL_LogPrintf("GAMETIME %i\n", ( (int)gpGlobals->time - (int)g_flGameTime) );
	
	for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		edict_t *pClient = INDEXENT(i);

		if(UTIL_IsPlayerValid(pClient))
		{
			PLAYER_CNX_STATS(pClient, &iPing, &iLoss);
			
			UTIL_LogPrintf("#%s# %i %i %i %i %i %i %i %i %i %i\n",
				GETPLAYERAUTHID(pClient),
				GETPLAYERUSERID(pClient),
				PlayerData[i].iScore,
				PlayerData[i].iFrags,
				PlayerData[i].iDeaths,
				PlayerData[i].iPlayerClass,
				PlayerData[i].iIconInfo,
				PlayerData[i].iTeam,
				(int)pClient->v.vuser4.z/100,
				(int)(gpGlobals->time - PlayerData[i].flConnectionTime),
				iPing);
		}
	}
	UTIL_LogPrintf("ENDUSERDATA\n");	
}

void HandleScoreInfo (void *buf)
{
	static int CurrentArg = 0;
	static int iPlayerIndex = 0;
	
	if(buf == NULL)
	{
		CurrentArg = 0;
		iPlayerIndex = 0;
		return;
	}
	
	switch(CurrentArg)
	{
	case 0:
		iPlayerIndex = M_INT(buf); 
		CurrentArg++;
		break;
	case 1://Score
		PlayerData[iPlayerIndex].iScore = M_INT(buf);
		CurrentArg++;
		break;
	case 2://Kills
		PlayerData[iPlayerIndex].iFrags = M_INT(buf);
		CurrentArg++;
		break;
	case 3://Deaths
		PlayerData[iPlayerIndex].iDeaths = M_INT(buf);
		CurrentArg++;
		break;
	case 4://Class
		PlayerData[iPlayerIndex].iPlayerClass = M_INT(buf);
		CurrentArg++;
		break;
	case 5:
		PlayerData[iPlayerIndex].iIconInfo = M_INT(buf);
		CurrentArg++;
		break;
	case 6://Team
		PlayerData[iPlayerIndex].iTeam = M_INT(buf);
		CurrentArg++;
		break;
	default:
		CurrentArg++;
		break;
	}
}

void GameDLLInit ( void )
{
	g_engfuncs.pfnAddServerCommand("advstatus", AdvStatus);

	RETURN_META(MRES_HANDLED);
}

BOOL ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[])
{
	memset(&PlayerData[ENTINDEX(pEntity)], 0, sizeof(playerdata_t));

	PlayerData[ENTINDEX(pEntity)].flConnectionTime = gpGlobals->time;

	RETURN_META_VALUE(MRES_HANDLED, TRUE);
}

void ClientDisconnect( edict_t *pEntity )
{
	memset(&PlayerData[ENTINDEX(pEntity)], 0, sizeof(playerdata_t));
	RETURN_META(MRES_HANDLED);
}

void pfnMessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed)
{
	MsgFunction = NULL;

	if(!g_bRegMsg)
	{
		SAFE_USER_MSG(gmsgScoreInfo,	  "ScoreInfo",  -1);
		SAFE_USER_MSG(gmsgCountdown,	  "Countdown",  -1);
		g_bRegMsg = true;
	}


	if(msg_type == gmsgScoreInfo) MsgFunction = HandleScoreInfo;
	if(msg_type == gmsgCountdown) g_flGameTime = gpGlobals->time;

	if(MsgFunction)
		(*MsgFunction)(NULL);

	RETURN_META(MRES_HANDLED);
}

void pfnWriteByte ( int iValue)
{
	if(MsgFunction)
	{
		(*MsgFunction)( (void *)&iValue);
		RETURN_META(MRES_HANDLED);
	}
	RETURN_META(MRES_IGNORED);
}

void pfnWriteShort ( int iValue)
{
	if(MsgFunction)
	{
		(*MsgFunction)( (void *)&iValue);
		RETURN_META(MRES_HANDLED);
	}
	RETURN_META(MRES_IGNORED);
}

void pfnMessageEnd( void )
{
	if(MsgFunction)
		(*MsgFunction)(NULL);

	MsgFunction = NULL;

	RETURN_META(MRES_HANDLED);
}

// Must provide at least one of these..
static META_FUNCTIONS gMetaFunctionTable = {
		NULL,			// pfnGetEntityAPI				HL SDK; called before game DLL
		NULL,			// pfnGetEntityAPI_Post			META; called after game DLL
		GetEntityAPI2,	// pfnGetEntityAPI2				HL SDK2; called before game DLL
		NULL,			// pfnGetEntityAPI2_Post		META; called after game DLL
		NULL,			// pfnGetNewDLLFunctions		HL SDK2; called before game DLL
		NULL,			// pfnGetNewDLLFunctions_Post	META; called after game DLL
		GetEngineFunctions,			// pfnGetEngineFunctions		META; called before HL engine
		NULL,			// pfnGetEngineFunctions_Post	META; called after HL engine
};

// Description of plugin
plugin_info_t Plugin_info = {
	META_INTERFACE_VERSION,	// ifvers
		"AdvStatusMM",	// name
		"1.1",		// version
		"6/24/2004",	// date
		"voogru <voogru@voogru.com>",	// author
		"http://www.voogru.com",	// url
		"ADVSTATUS",	// logtag
		PT_CHANGELEVEL,// (when) loadable
		PT_CHANGELEVEL,// (when) unloadable
};

// Global vars from metamod:
meta_globals_t *gpMetaGlobals;		// metamod globals
gamedll_funcs_t *gpGamedllFuncs;	// gameDLL function tables
mutil_funcs_t *gpMetaUtilFuncs;		// metamod utility functions

C_DLLEXPORT int Meta_Query(char *ifvers, plugin_info_t **pPlugInfo,	mutil_funcs_t *pMetaUtilFuncs) {
	*pPlugInfo=&Plugin_info;
	gpMetaUtilFuncs=pMetaUtilFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Attach(PLUG_LOADTIME now, META_FUNCTIONS *pFunctionTable, meta_globals_t *pMGlobals, gamedll_funcs_t *pGamedllFuncs) {
	gpMetaGlobals=pMGlobals;
	memcpy(pFunctionTable, &gMetaFunctionTable, sizeof(META_FUNCTIONS));
	gpGamedllFuncs=pGamedllFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Detach(PLUG_LOADTIME now, PL_UNLOAD_REASON reason) 
{
	return(TRUE);
}

void WINAPI GiveFnptrsToDll( enginefuncs_t* pengfuncsFromEngine, globalvars_t *pGlobals ) {
	memcpy(&g_engfuncs, pengfuncsFromEngine, sizeof(enginefuncs_t));
	gpGlobals = pGlobals;		
}

DLL_FUNCTIONS gFunctionTable;
C_DLLEXPORT int GetEntityAPI2( DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion ) 
{
	gFunctionTable.pfnGameInit = GameDLLInit;
	gFunctionTable.pfnClientConnect = ClientConnect;
	gFunctionTable.pfnClientDisconnect = ClientDisconnect;
	memcpy( pFunctionTable, &gFunctionTable, sizeof( DLL_FUNCTIONS ) );
	return(TRUE);
}



enginefuncs_t meta_engfuncs;
C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion ) 
{
	meta_engfuncs.pfnMessageBegin = pfnMessageBegin;
	meta_engfuncs.pfnWriteByte = pfnWriteByte;
	meta_engfuncs.pfnWriteShort = pfnWriteShort;
	meta_engfuncs.pfnMessageBegin = pfnMessageBegin;
	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
	return(TRUE);
}


